class Game extends ManageLang {
    constructor(father) {
        super('Game', father);
        super.addOtherScenes('GamingBoard');
        SPEED_GAME = 1;
    }

    create() {
        super.create();
        this.doLine = false;
        this.gm = new GamingBoard(this);
        if (this.father.scene.getIndex('GamingBoard') !== -1) {
            this.father.scene.remove('GamingBoard');
        }
        this.father.scene.add('GamingBoard', this.gm);
        this.scene.launch('GamingBoard');
        this.scene.pause('GamingBoard');

        this.selected = null;

        let c1 = new CategoryBar();
        this.damageRobot = new Bar(this, 200, 30, OFFX_MAP + WIDTH_MAP / 4, OFFY_MAP / 2, COLOR_ATTACK, c1, DAMAGE_BEGIN);
        this.speedReloadRobot = new Bar(this, 200, 30, OFFX_MAP + WIDTH_MAP / 4, 0, COLOR_SPEED_RELOAD, c1, SPEED_RELOAD_BEGIN);
        let c2 = new CategoryBar();
        this.speedRobot = new Bar(this, 200, 30, OFFX_MAP + WIDTH_MAP / 4 * 3, 0, COLOR_MOVE, c2, SPEED_BEGIN);
        this.rangeRobot = new Bar(this, 200, 30, OFFX_MAP + WIDTH_MAP / 4 * 3, OFFY_MAP / 2, COLOR_RANGE, c2, RANGE_BEGIN);

        this.createButton();

        let centerX = WIDTH_WINDOW / 4 * 3;
        let diffX = WIDTH_WINDOW / 10;
        this.tree = new RectangleNode(centerX, HEIGHT_WINDOW / 10, this, 'waria');
        this.tree.addRect(new RectangleNode(centerX - 2 * diffX, HEIGHT_WINDOW / 3, this, 'move', true, false));
        this.tree.addRect(new RectangleNode(centerX + diffX, HEIGHT_WINDOW / 3, this, 'move', true));
        this.tree.addRect(new RectangleNode(centerX, HEIGHT_WINDOW / 3, this, 'attack', 1 / 3));


        let condition = new RectangleNode(centerX - diffX, HEIGHT_WINDOW / 3, this, 'condition', true, true, false, 1 / 3, 0);
        condition.addRect(new RectangleNode(centerX, HEIGHT_WINDOW / 1.5, this, 'move', false));
        condition.addRect(new RectangleNode(centerX - diffX, HEIGHT_WINDOW / 1.5, this, 'attack'))
        this.tree.addRect(condition);

        this.gm.addRobot(new Robot(WIDTH_MAP * 0.1, WIDTH_MAP * 0.1, WIDTH_MAP * 0.9, WIDTH_MAP * 0.9, 0x6666ff, this.gm, MYSELF));
        this.gm.addRobot(new Robot(WIDTH_MAP * 0.1, WIDTH_MAP * 0.1, WIDTH_MAP * 0.2, WIDTH_MAP * 0.1, 0xff0000, this.gm));
        this.gm.addRobot(new Robot(WIDTH_MAP * 0.1, WIDTH_MAP * 0.1, WIDTH_MAP * 0.1, WIDTH_MAP * 0.9, 0xff0000, this.gm));


        this.input.keyboard.on('keydown-DELETE', () => {
            this.clickBin();
        });
    }

    update(time, delta) {
        super.update(time, delta);
        if (this.doLine)
            this.tree.updateLine();
    }

    finish() {
        console.log("FINISH");
        let winner;
        if (this.gm.winner()) {
            winner = new Winner(this.father);
        } else {
            winner = new Looser(this.father);
        }
        this.father.scene.add('Finish', winner);

        this.scene.pause('Game');

        this.scene.launch('Finish');
    }


    clickNode() {
        console.log("NODE");
        this.tryClick();
        this.selected = this.tree.getSelected();

        if (this.selected === null)
            return;

        if (this.selected !== this.tree) {
            this.pencil.setVisible(true);
            this.bin.setVisible(true);
        }
        if (this.selected.canAddNode) {
            this.plus.setVisible(true);
        }
    }

    doDrag(pointer, target, dragX, dragY) {
        target.setX(dragX);
        target.setY(dragY);
    }

    dragStart() {
        console.log("START DRAG");

        this.doLine = true;
        this.pauseScene();
    }

    dragEnd() {
        console.log("END DRAG");

        this.doLine = false;
    }


    pauseScene() {
        this.linePlay = 0;
        this.changeFrame(this.play, 0);

        this.gm.pause();
    }

    resume() {
        this.linePlay = 1;

        let sortDesc = (a, b) => a.getX() - b.getX();
        this.tree.sort(sortDesc);
        this.gm.modifyNodes(this.tree.getNode());
        this.gm.modifyValue(this.speedRobot.value, this.rangeRobot.value, this.speedReloadRobot.value, this.damageRobot.value);

        this.gm.resume();
    }


    changeFrame(btn, column = 0, line = 0) {
        btn.setFrame(column + line * 2);
    }

    createButton() {
        let x = WIDTH_MAP + WIDTH_WINDOW / 10;
        let y = WIDTH_MAP + HEIGHT_WINDOW / 10 + 5;
        let diffX = (HEIGHT_WINDOW / 10);
        let width = HEIGHT_WINDOW / 15;

        let back = this.add.sprite(10, 10, 'back').setOrigin(0, 0).setInteractive();

        this.play = this.add.sprite(x, y, 'play').setOrigin(1, 0).setInteractive();
        let speed = this.add.sprite(x - diffX, y, 'speed').setOrigin(1, 0).setInteractive();

        this.pencil = this.add.sprite(x - diffX * 2, y, 'pencil').setOrigin(1, 0).setInteractive().setVisible(false);
        this.bin = this.add.sprite(x - diffX * 3, y, 'bin').setOrigin(1, 0).setInteractive().setVisible(false);
        this.plus = this.add.sprite(x - diffX * 4, y, 'plus').setOrigin(1, 0).setInteractive().setVisible(false);

        let mute = this.add.sprite(x - diffX * 5, y, 'mute').setOrigin(1, 0).setInteractive();

        this.linePlay = 0;
        if (SPEED_GAME === 1) {
            this.lineSpeed = 0;
        } else {
            this.lineSpeed = 1;
        }
        this.changeFrame(speed, 0, this.lineSpeed);
        if (this.father.sound.mute) {
            this.lineMute = 1;
        } else {
            this.lineMute = 0;
        }
        this.changeFrame(mute, 0, this.lineMute);

        back.displayHeight = back.displayWidth = width;

        this.play.displayHeight = this.play.displayWidth = width;
        speed.displayHeight = speed.displayWidth = width;
        mute.displayHeight = mute.displayWidth = width;
        this.pencil.displayHeight = this.pencil.displayWidth = width;
        this.bin.displayHeight = this.bin.displayWidth = width;
        this.plus.displayHeight = this.plus.displayWidth = width;

        this.play.on('pointerout', () => this.changeFrame(this.play, 0, this.linePlay));
        this.play.on('pointerover', () => this.changeFrame(this.play, 1, this.linePlay));
        speed.on('pointerout', () => this.changeFrame(speed, 0, this.lineSpeed));
        speed.on('pointerover', () => this.changeFrame(speed, 1, this.lineSpeed));
        mute.on('pointerout', () => this.changeFrame(mute, 0, this.lineMute));
        mute.on('pointerover', () => this.changeFrame(mute, 1, this.lineMute));
        this.creatBtnOutAndOver(back);
        this.creatBtnOutAndOver(this.pencil);
        this.creatBtnOutAndOver(this.bin);
        this.creatBtnOutAndOver(this.plus);

        this.play.on('pointerdown', () => this.clickPlay());
        speed.on('pointerdown', () => this.clickSpeed(speed));
        back.on('pointerdown', () => this.clickBack());
        mute.on('pointerdown', () => this.clickMute(mute));
        this.pencil.on('pointerdown', () => this.clickPencil());
        this.bin.on('pointerdown', () => this.clickBin());
        this.plus.on('pointerdown', () => this.clickPlus());

        this.input.on('pointerdown', () => this.clickNode());
        this.input.on('drag', this.doDrag);
        this.input.on('dragstart', () => this.dragStart());
        this.input.on('dragend', () => this.dragEnd());
    }

    creatBtnOutAndOver(btn) {
        btn.on('pointerout', () => this.changeFrame(btn, 0));
        btn.on('pointerover', () => this.changeFrame(btn, 1));
    }

    clickPlay() {
        if (this.scene.isPaused('GamingBoard'))
            this.resume();
        else
            this.pauseScene();

        this.changeFrame(this.play, 1, this.linePlay);
    }

    clickSpeed(btn) {
        if (SPEED_GAME === 1) {
            SPEED_GAME = 2;
            this.lineSpeed = 1;
        } else {
            SPEED_GAME = 1;
            this.lineSpeed = 0;
        }

        this.changeFrame(btn, 1, this.lineSpeed);
    }

    clickPencil() {
        console.log("PENCIL");


        this.changeFrame(this.pencil, 0);
    }

    clickBin() {
        if (this.selected !== null) {
            console.log("BIN");
            this.tree.delete(this.selected);

            this.changeFrame(this.bin, 0);
            this.tryClick();
        }
    }

    clickPlus() {
        console.log("PLUS");

        this.father.scene.add('PlusNode', new PlusNode(this.father, this, this.selected));

        this.scene.launch('PlusNode');

        this.changeFrame(this.plus, 0);

        this.scene.pause('Game');
    }

    clickBack() {
        console.log("BACK");

        this.father.scene.stop('GamingBoard');
        this.father.scene.remove('GamingBoard');
        this.father.scene.stop('Game');
        this.father.scene.start('Type');
    }

    clickMute(btn) {
        console.log("MUTE");

        if (this.father.sound.mute) {
            this.father.sound.mute = false;
            this.lineMute = 0;
        } else {
            this.father.sound.mute = true;
            this.lineMute = 1;
        }
        this.changeFrame(btn, 1, this.lineMute);
    }

    tryClick() {
        this.pencil.setVisible(false);
        this.bin.setVisible(false);
        this.plus.setVisible(false);
        if (this.selected != null) {
            this.selected.deselect();
            this.selected = null;
        }
    }
}